function [NewModel]=ExpectationMaximization(OldModel)
% Coded by Ezequiel Lpez-Rubio, April 2012.
% Apply the EM method (expectation maximization)
% Only the mean vector and the covariance matrix are optimized,
% while the a priori probabilities are kept uniform

NewModel=OldModel;

% Input space dimension
Dimension=size(NewModel.Samples,1);
for NdxGaussian=1:length(NewModel.Covariances)
    NewModel.NumPrincipalDirections{NdxGaussian}=Dimension;
    NewModel.Sigma2{NdxGaussian}=0;
end

KeepOn=1;
while KeepOn
    for NdxGaussian=1:length(NewModel.Covariances)
        NewModel.InvCovariances{NdxGaussian}=inv(NewModel.Covariances{NdxGaussian});
        NewModel.NormCovariances{NdxGaussian}=det(NewModel.Covariances{NdxGaussian});
        for NdxSample=1:size(NewModel.Samples,2)
            SampleProb(NdxGaussian,NdxSample)=...
                LocalCompleteGaussian(NewModel,NewModel.Samples(:,NdxSample),NdxGaussian);
        end        
    end
    for NdxSample=1:size(NewModel.Samples,2)
        % Normalize probabilities
        SumProbs=sum(SampleProb(:,NdxSample));
        SampleProb(:,NdxSample)=SampleProb(:,NdxSample)/SumProbs;
    end
    % Now SampleProb(n,m) is the probability that sample m
    % comes from the n-th Gaussian
    % Next we recompute the mean vectors and the covariance matrices
    Variations=[];
    for NdxGaussian=1:length(NewModel.Covariances)
        MeanVector=zeros(size(NewModel.Means{NdxGaussian}));

        SumProbs2=sum(SampleProb(NdxGaussian,:));        
        for NdxSample=1:size(NewModel.Samples,2)
            MeanVector=MeanVector+...
                SampleProb(NdxGaussian,NdxSample)*NewModel.Samples(:,NdxSample);
        end
        MeanVector=MeanVector/SumProbs2;
        CovMatrix=zeros(size(NewModel.Covariances{NdxGaussian}));        
        for NdxSample=1:size(NewModel.Samples,2)      
            VectorDif=NewModel.Samples(:,NdxSample)-MeanVector;
            CovMatrix=CovMatrix+...
                SampleProb(NdxGaussian,NdxSample)*VectorDif*VectorDif';
        end
        CovMatrix=CovMatrix*SumProbs2;
        Variations(end+1)=norm(CovMatrix-NewModel.Covariances{NdxGaussian})/...
            norm(NewModel.Covariances{NdxGaussian});
        % Update values
        NewModel.Covariances{NdxGaussian}=CovMatrix;
        NewModel.Means{NdxGaussian}=MeanVector;
    end
    % Continue while the covariance matrices change
    KeepOn= (mean(Variations)<0.1);
end
    
