function [NewModel]=MergeCovariances(OldModel,NonSmoothedModel)
% Coded by Ezequiel Lpez-Rubio, April 2012.
% Combine the covariance matrices from several samples according to a weight for each
% sample. This produces a smoothing effect.
% There can be overlapping among clusters. The sample weights are
% normalized by clusters, that is:
% sum(OldModel.SampleWeights(NdxCluster,:))==1 for all NdxCluster

NewModel=OldModel;

% Compute the correlation matrices, R
Correlations=zeros(size(NonSmoothedModel.Samples,1),size(NonSmoothedModel.Samples,1),...
    size(NonSmoothedModel.Samples,2));
for NdxSample=1:size(NonSmoothedModel.Samples,2)
    Correlations(:,:,NdxSample)=NonSmoothedModel.Covariances{NdxSample}+...
        NonSmoothedModel.Means{NdxSample}*NonSmoothedModel.Means{NdxSample}';
end
Means=cell2mat(NonSmoothedModel.Means);

for NdxCluster=1:NewModel.NumClusters
    NewModel.Means{NdxCluster}=zeros(size(NonSmoothedModel.Samples,1),1);
    NdxNonNullWeights=find(NewModel.SampleWeight(NdxCluster,:));
    MyMean=sum(Means(:,NdxNonNullWeights).*...
        kron(NewModel.SampleWeight(NdxCluster,NdxNonNullWeights),ones(size(NonSmoothedModel.Samples,1),1)),...
        2);
   MyCorrelation=sum(Correlations(:,:,NdxNonNullWeights).*...
        reshape(kron(NewModel.SampleWeight(NdxCluster,NdxNonNullWeights),ones(size(NonSmoothedModel.Samples,1))),...
        [size(NonSmoothedModel.Samples,1) size(NonSmoothedModel.Samples,1) numel(NdxNonNullWeights)]),...
        3);
    NewModel.Means{NdxCluster}=MyMean;
    NewModel.Covariances{NdxCluster}=MyCorrelation-...
        NewModel.Means{NdxCluster}*NewModel.Means{NdxCluster}';
end

    