function [Model]=TrainSmoothParzenWindows(Samples,VarianceAmount,NumNeighbors,NumClusters,PhiParameter,GammaParameter,Method)
% Coded by Ezequiel Lpez-Rubio, April 2012.
% Inputs:
%   Samples=Matrix where each column is an input sample
% Note: this file uses the fcm function (fuzzy k-means), from the Fuzzy Toolbox

Model.NumNeighbors=NumNeighbors;
Model.NumClusters=NumClusters;
Model.Samples=Samples;
Model.GammaParameter=GammaParameter;
Model.PhiParameter=PhiParameter;
Model.VarianceAmount=VarianceAmount;
Model.Method=Method;

% Obtain the non smoothed model
NonSmoothedModel=PreTrainSmoothParzenWindows(Samples,VarianceAmount,NumNeighbors,GammaParameter);

if strcmp(Method,'fuzzy')==1
    % Fuzzy k-means (avoid NaNs in GradeMembership)
    KeepOn=1;
    NumIterations=100;
    while KeepOn
        [Centroids,GradeMembership] = fcm(Samples',NumClusters,[NaN NumIterations NaN 0]);
        KeepOn=sum(sum(find(1-isfinite(GradeMembership))))~=0;
        NumIterations=NumIterations-9;
    end
    % Compute the weights for the mixture of Gaussians
    SumMemberships=sum(GradeMembership,2);
    for NdxCluster=1:NumClusters
        Model.SampleWeight(NdxCluster,:)=GradeMembership(NdxCluster,:)./SumMemberships(NdxCluster);
    end
else
    if strcmp(Method,'dist')==1
        Indices=randperm(size(Samples,2));
        Indices=Indices(1:NumClusters);
        Model.SampleWeight=zeros(NumClusters,size(Samples,2));
        for NdxSample=1:numel(Indices)
            Distances=distanceMEX(Samples(:,Indices(NdxSample)),Samples);
            Omega=exp(-(Distances.^2)/(PhiParameter^2));
            SumOmegas=sum(Omega);
            Model.SampleWeight(NdxSample,:)=Omega./SumOmegas;
        end
        
    else
        error('Unrecognized method')
    end
    
end




% Combine the covariance matrices (smoothing)
CombinedModel=MergeCovariances(Model,NonSmoothedModel);

if VarianceAmount>0
    % Apply the explained variance method
    Model=ExplainedVariance(CombinedModel);
else
    % Apply the EM method (expectation-maximization)
    Model=ExpectationMaximization(CombinedModel);
end


d=size(Samples,1);
NumGaussians=length(Model.Covariances);
Model.NumParameters=2*NumGaussians-1; % Pi and Sigma2
for NdxGaussian=1:NumGaussians
    q=Model.NumPrincipalDirections{NdxGaussian};
    % Eigenvectors and mu
    Model.NumParameters=Model.NumParameters+...
        (d*(q+1)+1-q*(q-1)/2);
end
